/* Encoder Library
   http://www.pjrc.com/teensy/td_libs_Encoder.html
   Brett Oliver http://www.brettoliver.org.uk/Height_Measure/Digital_Stadiometer_quadrature_encoder.htm
   Height measure using a 600P/R AB 2-Phases Photoelectric Incremental Rotary Encoder 5V-24V Shaft 6mm 
   This example code is in the public domain.
   v7 add dotmatrix
   v8 add move to top to start
   v9 working top to start
   v10 added restart 2nd measure when height bar goes to top again
   v11 added mem store and clear
   v12 add display of change in mm (not working when disp diff is on)
   v13 added brightness not working
   v14 add matrix message to startup working
   v14a working
   v18 working
   v19 working can't save height after disp height change
   v22 save removed reset working
   v26 brightness working
   v27 code cleanup
   v29 changed M to mm
   v30 added neg encoder value correction
   v31 added mmStep to calibrate mm per step
*/

#include <Encoder.h>
// Change these two numbers to the pins connected to your encoder.
//   Best Performance: both pins have interrupt capability
//   Good Performance: only the first pin has interrupt capability
//   Low Performance:  neither pin has interrupt capability
Encoder myEnc(2, 3);  ////   Best Performance: both pins have interrupt capability
//   avoid using pins with LEDs attached




//Include Libraries and Pins for DOT Matric displays
#include <MD_Parola.h>
#include <MD_MAX72xx.h>
#include <SPI.h>
#define HARDWARE_TYPE MD_MAX72XX::FC16_HW
#define MAX_DEVICES 8
#define CLK_PIN   13
#define DATA_PIN  11
#define CS_PIN    10

// Hardware SPI connection
MD_Parola P = MD_Parola(HARDWARE_TYPE, CS_PIN, MAX_DEVICES);

// sets scrolling direction if slider in middle at start
uint8_t scrollSpeed = 25;    // default frame delay value
textEffect_t scrollEffect = PA_SCROLL_LEFT;
textPosition_t scrollAlign = PA_LEFT;  // how to align the text
textEffect_t displayClear = PA_SCROLL_UP;
uint16_t scrollPause = 1500; // in milliseconds
#define  BUF_SIZE  75  // Maximum of 75 character


int intensity = 5;
int intensitySw = 0;
int intensityDown = 4;
int intensityDownSw = 0;

float mmStep = 0.0565347; //mms moved per encoder step
int lock = 0; //locks measuring reset to unlock
int Reset = A2;// output to reset Arduino when disp change/measure pressed together
int measureLED = A1;//Indicates measuring
int pd_brightness = 2;  //Dot matrix LED brightness
int measure = 7; //Pin to send measurement to NANO
int measureSw = 0; //Measure Height
int startup = 1; // Locks on the measurment loop once measure is pressed
int reseton = 0; // Locks on the reading from encodor loop
int resetHeight = 8;
int resetHeightSw = 0;
int memstop = 0; // stop multiple write to EEPROM
int resetSw = 0; //part of reset switch
int resetNANO   = 6; //resets the NANO when pressed
long oldPosition = -999;
long newPosition;
float distance; // Stores calculated distance mm
int distanceInt;// Stores distance in mm as an int (no fractions)
float distanceM; // Stores calculated distance in metres
float distancefeet; //height in feet
int distanceINTfeet; //whole mumber of feet
float distanceinches; //height in inches
float inches; //inches

char testStr[10] = { "" };
char startupStr[0] = { "" }; // [0] is start character
String heightStr = "";
String inchStr = "";
String footStr = "";
String startStr = "";


void setup() {
  digitalWrite(Reset, HIGH); //sets reset to high to prevent NANO reseting on startup
  delay(200);
  pinMode(Reset, OUTPUT);
  pinMode(intensity, INPUT);
  pinMode(intensityDown, INPUT);
  pinMode(measure, INPUT);
  pinMode(resetHeight, INPUT);
  pinMode(resetNANO, INPUT);
  pinMode(measureLED, OUTPUT);
  

  Serial.begin(9600);

  P.begin();
  P.setIntensity(3);


}



void loop() {
  resetHeightSw = digitalRead(resetHeight);   // read the resetHeight button when height bar is topmost position
  measureSw = digitalRead(measure);   // read the measure button
  
  
  resetSw = digitalRead(resetNANO);   // read the reset button

  //NANO reset
  if (resetSw == HIGH)// resets the NANO to take another reading
  {

    digitalWrite(Reset, LOW);
    Serial.println("RESET ");

  }

/*
  // LED brightness up
  intensitySw = digitalRead(intensity);   // read the intensity up button

  if (intensitySw == HIGH)    // if the button was released, change the LED state
  {
    pd_brightness++;
    delay(200);
    P.setIntensity(pd_brightness);
    if (pd_brightness > 15)
    {
      pd_brightness = 0;
    }
    Serial.print("Intensity Up ");
    Serial.println(pd_brightness);
  }

  // LED brightness down
  intensityDownSw = digitalRead(intensityDown);   // read the intensity down button

  if (intensityDownSw == HIGH)    // if the button was released, change the LED state
  {
    pd_brightness--;
    delay(200);
    P.setIntensity(pd_brightness);
    if (pd_brightness < 0)
    {
      pd_brightness = 15;
    }
    Serial.print("Intensity Down ");
    Serial.println(pd_brightness);
  }

*/


  //Reads measurements from the encoder
  if (resetHeightSw == 1 || reseton == 1 && measureSw == 0 && lock == 0 )//on start reseton is 0 so waits for height bar to be returnes to top
  {
    reseton = 1;
    digitalWrite(measureLED, HIGH);//LED turns On & Off while height bar is moving
    newPosition = myEnc.read();

    if (newPosition != oldPosition) {
      oldPosition = newPosition;
      digitalWrite(measureLED, LOW); //LED turns On & Off while height bar is moving
      Serial.println(-1*newPosition);// remove -1 if encoder give +ve readings
      Serial.print("Length ");
      Serial.print(newPosition * mmStep);  
      Serial.println("mm");
      //Loop continues on (until the measure switch is pressed) and updates each time the rotary encoder moves



    }
  }

  // Sends reading from encoder when the Measure button pressed
  else if ( measureSw == 1 || startup == 0 ) // when measure switch is pressed measure loop stops and a measure is taken
  {
    reseton = 0;
    digitalWrite(measureLED, LOW);
    newPosition = myEnc.read();
    newPosition = newPosition * -1; // comment out if encoder is +ve
    distance = 1913 - (newPosition * mmStep); //distance in mm. 1913 is diatance to bottom of height bar with microswitch operated
   // distance = distance * -1; // comment out if encoder readings are +ve
    distanceM = distance / 1000; //distance in metres



    distancefeet = distanceM * 3.281;  // distance in feet inc fractions of feet
    distanceINTfeet = distancefeet; //Distance in feet no fractions
    distanceinches = distancefeet - distanceINTfeet;
    inches = distanceinches * 12;
    distanceInt = distance; // distance no float

   


    footStr = String(distanceINTfeet);

    inchStr = String(distanceinches * 12, 1);

    heightStr = String(distance, 2) + "mm   " + footStr + " foot " + inchStr + " in ";

    heightStr.toCharArray(testStr, 75);

 // LED brightness up
  intensitySw = digitalRead(intensity);   // read the intensity up button
if (intensitySw == HIGH)    // if the button was released, change the LED state
  {
    pd_brightness++;
    delay(200);
    P.setIntensity(pd_brightness);
    if (pd_brightness > 15)
    {
      pd_brightness = 0;
    }
    Serial.print("Intensity Up ");
    Serial.println(pd_brightness);
  }

  // LED brightness down
  intensityDownSw = digitalRead(intensityDown);   // read the intensity down button

  if (intensityDownSw == HIGH)    // if the button was released, change the LED state
  {
    pd_brightness--;
    delay(200);
    P.setIntensity(pd_brightness);
    if (pd_brightness < 0)
    {
      pd_brightness = 15;
    }
    Serial.print("Intensity Down ");
    Serial.println(pd_brightness);
  }


     //Dot matrix scroll commands Parola library
    if (P.displayAnimate())
      P.displayText(testStr, PA_CENTER, scrollSpeed, scrollPause, PA_SCROLL_LEFT, PA_SCROLL_UP);


    startup = 0;// allows measuring once per press
    lock = 1; // locks measuring after one run reset to restart
  }



  else
  {


  // noIntensity = 1;// disables intensity at start message
    //Startup Dot Matrix message and scroll if no buttons pressed on startup  Parola library
    startStr = "Move Height Bar To Top, Slide Down To Head Then Press Measure";
    startStr.toCharArray(startupStr, 75);
    if (P.displayAnimate())
      P.displayText(startupStr, PA_CENTER, scrollSpeed, scrollPause, PA_SCROLL_LEFT, PA_SCROLL_UP);
   
  }

}
